%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed A:
% Stochastic OPF in Presence of Renewable Energy and Controllable Loads
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
%
% Employing MATPOWER as underlying power flow and basic Particle Swarm  
% Evoluation (PSO) algorithm as optimization engine, you can use the 
% test bed declarations as the template shown below.
%
% Results will be buffered and agglomerated automatically for storage to 
% formatted ASCII-files. Refer to problem definitions and implementation 
% guidelines for details.
%
% Note:
% This implementation has been tested using various MATLAB versions and
% hardware platforms. Feel free to contact us in case of incompatibilities.
% A MATPOWER installation must be on the MATLAB search path.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
clc
global proc
global ps
global mpc
global res
%addpath ('matpower6.0b1')
algorithm_name='EE_CMAES';
algorithm_hd=str2func(algorithm_name);
test_bed_OPF_hd=str2func('test_bed_OPF');
system=57; % please note that the system only can be 57, 
           % the test bed A of 2018 Stochastic OPF Competition only apply for this system
pop_size=100;
run_in_parallel=0; % please note that the parallel run functionallity is not
                   % valid for the 2018 Stochastic OPF Competition
refresh=500;
show_lf_info=0;      

args{1}=system;
args{2}=show_lf_info;
args{3}=pop_size;
args{4}=refresh;
args{5}=algorithm_name;
args{6}=run_in_parallel;
args{7}=[];
args{8}=[];

global ScenarioType
%%CASES (to run the wanted cases please uncomment the respective line)
%1
%ScenarioType=1; % UNCOMMENT FOR CASE 1
%2
%ScenarioType=2; % UNCOMMENT FOR CASE 2
%3
%ScenarioType=3; % UNCOMMENT FOR CASE 3
%4
%ScenarioType=4; % UNCOMMENT FOR CASE 4
%5
ScenarioType=5; % UNCOMMENT FOR CASE 5
fprintf('Case_ %d \n', ScenarioType)

i=1; % please note that i must be 1, since it is only considered the Cost 
     % optimization not the losses
stop_test_case=0;

if i==1 && system==57

        while ~stop_test_case
            i=i+1;
            j=0;
            stop_scenario=0;
            while ~stop_scenario
                j=j+1;
                % Initialization of power system and optimization related
                % quantities
                [stop_test_case,stop_scenario,err,obs]=test_bed_OPF_hd(i,j,1,args);
                args{7}=stop_test_case;
                args{8}=stop_scenario;
                if ~err
                    % To prematurely stop running the procedure
                    % in terms of independent trials, you may use
                    % a statement as the following (change proc.n_run to the wanted runs).
                    % This will be without effect on output of 
                    % intermediate results to formatted ASCII files.
                     %%%%%%%%%%%%%
                    proc.n_run=12;
                    Runed=proc.n_run;
                     %%%%%%%%%%%%%
                    for k=1:proc.n_run  
                        test_bed_OPF_hd(i,j,k,args);
                        feval(algorithm_hd,test_bed_OPF_hd,i,j,k,args);
                        fprintf('Run %d finished.\n',k);
                    end
                    if Runed ~=12
                        fprintf('Please note that you did not run the required 12 runs');
                    else
                        test_bed_OPF_hd(i,j,987,args);
                    end
                end
            end
        end

else
    
    display('The Test Bed A only apply for the 57 buses system and OARPD (cost minimization)')

end
